// Types.h
//------------------------------------------------------------------------------
#pragma once
#ifndef CORE_ENV_TYPES_H
#define CORE_ENV_TYPES_H

typedef unsigned char		uint8_t;
typedef signed char			int8_t;
typedef unsigned short		uint16_t;
typedef signed short		int16_t;
typedef unsigned int		uint32_t;
typedef signed int			int32_t;
typedef unsigned long long	uint64_t;
typedef signed long long	int64_t;

#define UINT16_MAX (uint16_t(0xFFFF))
#define UINT32_MAX (uint32_t(0xFFFFFFFF))

#define KILOBYTE (1024)
#define MEGABYTE (1024 * 1024)

#define UNUSED( x )

#define THREAD_LOCAL __declspec( thread )

#define NO_INLINE __declspec( noinline )
#define FORCE_INLINE __forceinline

#ifdef __APPLE__
    #include <stddef.h> // needed for size_t
#endif

#ifndef intptr_t
	#if defined( WIN64 )
		typedef int64_t		intptr_t;
		typedef uint64_t	uintptr_t;
	#elif defined( WIN32 )
		typedef int32_t		intptr_t;
		typedef uint32_t	uintptr_t;
	#endif
#endif
#ifndef size_t
	#if defined( WIN64 )
		typedef uint64_t	size_t;
	#elif defined( WIN32 )
		typedef uint32_t	size_t;
	#endif
#endif

#if defined( __WINDOWS__ ) && defined( __clang__ )
	#define __w64
#endif

#ifndef nullptr
	#define nullptr (0)
#endif

#ifndef LONGLONG
	typedef long long LONGLONG;
#endif

// Warning disabling
//------------------------------------------------------------------------------
#if defined( WIN32 ) || defined( WIN64 )
    #define PRAGMA_DISABLE_PUSH_MSVC( num ) __pragma(warning(push))			\
                                            __pragma(warning(disable:num))
    #define PRAGMA_DISABLE_POP_MSVC         __pragma(warning(pop))
#else
    // Do nothing on non-msvc platforms
    #define PRAGMA_DISABLE_PUSH_MSVC( num )
    #define PRAGMA_DISABLE_POP_MSVC
#endif
#if defined( __clang__ )
	#define PRAGMA_DISABLE_PUSH_CLANG( w )	__pragma( clang diagnostic push ) \
											__pragma( clang diagnostic ignored w )
	#define PRAGMA_DISABLE_POP_CLANG		__pragma( clang diagnostic pop )
#else
	#define PRAGMA_DISABLE_PUSH_CLANG( w )
	#define PRAGMA_DISABLE_POP_CLANG
#endif
//------------------------------------------------------------------------------
#endif // CORE_ENV_TYPES_H
